'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var HTTP = 'http:',
    HTTPS = 'https:',
    url = require('url'),
    zlib = require('zlib'),
    http = require('http');

var HttpResolver = function () {
  function HttpResolver(state, href, parent) {
    _classCallCheck(this, HttpResolver);

    this.state = state;
    this.href = href;
    this.parent = parent;
    this.uri = url.parse(href);

    this._file = null;
  }

  _createClass(HttpResolver, [{
    key: 'getFileContents',


    /**
     *  Allows resolver implementations to load file content from a remote 
     *  resource.
     */
    value: function getFileContents(cb) {
      var called = false;

      function done(err, buf) {
        /* istanbul ignore next: guard against multiple events firing */
        if (called) {
          return;
        }
        cb(err, buf);
        called = true;
      }

      // ensure we are resolved relative to a parent
      this.uri = url.parse(this.getCanonicalPath());

      if (!this.uri.protocol) {
        return cb(new Error('https resolver attempt to load with no protocol'));
      }

      var pth = this.uri.pathname;

      var uri = url.format(this.uri),
          options = {
        hostname: this.uri.hostname,
        port: this.uri.port,
        path: pth,
        headers: {
          'Accept': 'text/html'
        }
      };

      this.getDefaultPort(this.uri.protocol, options);

      var req = http.get(options, function (res) {
        // expecting 200 response
        if (res.statusCode !== 200) {
          return done(new Error('unexpected status code ' + res.statusCode + ' from ' + uri));
        }

        var contentType = res.headers['content-type'],
            encoding = res.headers['content-encoding'];

        var gzip = false;

        if (!/text\/html/.test(contentType)) {
          return done(new Error('unexpected content type ' + contentType));
        }

        if (encoding && ~encoding.indexOf('gzip')) {
          gzip = true;
        }

        var buf = new Buffer(0);

        // consume response body
        res.resume();

        res.on('data', function (chunk) {
          buf = Buffer.concat([buf, chunk], buf.length + chunk.length);
        });

        // handle response error
        res.once('error', function (err) {
          /* istanbul ignore next: tough to mock repsonse stream error */
          done(err);
        });

        res.once('end', function () {
          if (gzip) {
            zlib.gunzip(buf, function (err, contents) {
              /* istanbul ignore next: not going to mock zlib deflate error */
              if (err) {
                return done(err);
              }
              done(null, contents);
            });
          } else {
            done(null, buf);
          }
        });
      });

      // handle request error
      req.once('error', function (err) {
        done(err);
      });
    }

    /**
     *  Allows resolvers for remote protocols to fetch resources from 
     *  the network.
     */

  }, {
    key: 'fetch',
    value: function fetch(cb) {
      cb();
    }

    /**
     *  Called after fetch has been invoked so that the resolver may return 
     *  a new filesystem path for downloaded content.
     */

  }, {
    key: 'getResolvedPath',
    value: function getResolvedPath() {
      return this.getCanonicalPath();
    }

    /**
     *  Get a canonical path for the URL reference, used to determine if the 
     *  resource has already been processed.
     */

  }, {
    key: 'getCanonicalPath',
    value: function getCanonicalPath() {
      // no scheme with a parent, resolve relative to the parent
      if (!this.uri.protocol && this.parent && this.parent.file) {
        return url.resolve(this.parent.file, this.href);
      }

      // TODO: use url.format() ?

      // should be an absolute HTTP/HTTPS URL
      return this.href;
    }
  }, {
    key: 'getDefaultPort',
    value: function getDefaultPort(protocol, options) {
      if (!options.port) {
        if (protocol === HTTP) {
          options.port = 80;
        } else {
          options.port = 443;
        }
      }
      return options;
    }
  }, {
    key: 'file',
    set: function set(val) {
      this._file = val;
    },
    get: function get() {
      return this._file;
    }
  }]);

  return HttpResolver;
}();

HttpResolver.HTTP = HTTP;
HttpResolver.HTTPS = HTTPS;

/**
 *  Resolver for the default http:// and https:// protocols.
 */
function file() /*state, conf*/{
  return function (registry) {
    registry.register(HTTP, HttpResolver);
    registry.register(HTTPS, HttpResolver);
  };
}

file.Resolver = HttpResolver;

module.exports = file;